@@ -27,7 +27,6 @@ import os
 import sys
 from getopt import GetoptError, gnu_getopt
 from getpass import getpass, getuser
-from tempfile import mktemp
 
 from pstorelib import VERSION_STRING
 from pstorelib.bytes import FileWithoutTrailingEnter, sendfile
@@ -37,10 +36,6 @@ from pstorelib.exceptions import (PStoreException, UserError, NotAllowed,
 from pstorelib.server import Backend
 
 
-# Optional. If they don't exist, you won't notice their features.
-SSH_ASKPASS_APP = '/usr/local/bin/ssh_echopass'
-SSH_ASKPASS_PRELOAD = '/usr/local/lib/ssh_askpass.so'
-
 ALPHA = ('ABCDEFGHIJKLMNOPQRSTUVWXYZ'
          'abcdefghijklmnopqrstuvwxyz')
 NUM = '0123456789'
@@ -99,24 +94,9 @@ class PStoreInterface(object):
                                         allowed_only=allowed_only,
                                         verbose=verbose)
 
-    def get_password(self, identifier):
+    def get_object(self, identifier):
         # Get object.
-        object = self.backend.get_object(identifier)
-        # The object fetched is already quite verbose. The password might be
-        # in there already.
-        property = self.PASSWORD_PROPERTY
-        try:
-            info = object['properties'][property]     # KeyError if not
-            decrypt_with = info['data'].decrypt_with  # AttributeError if None
-        except (AttributeError, KeyError):
-            fp = self.propget(identifier, property)
-        else:
-            if info['enctype'] == 'none':
-                sys.stderr.write('WARNING: password as public property?\n')
-            fp = decrypt_with()
-
-        password = fp.read()
-        return password, object
+        return self.backend.get_object(identifier)
 
     def get_properties(self, objectid, allowed_only=True):
         '''Get the available properties, possibly with contents.'''
@@ -150,7 +130,7 @@ class PStoreInterface(object):
         for property, info in object['properties'].items():
             if info['enctype'] == 'none':
                 continue
-            properties[property] = self.propget(
+            properties[property], enctype = self.propget(
                 objectid, property.encode('ascii'))
 
         # FIXME: there should be a way to denote that we are not updating the
@@ -354,30 +334,6 @@ def parse_options(args):
         elif option in ('--verbose', '-v'):
             config['verbose'] = True
 
-        # Evil hacks to allow the password input into ssh directly
-        elif option in ('-l',):
-            # XXX: move this to the ssh_host_hack bit..? or rename ssh_host to
-            # ssh_arg or something..
-            # Assume args[1] is the machine name here
-            machine_name = args[1]
-            # "x" (or "-") => use machine_name
-            if arg in ('-', 'x') and len(args) > 1:
-                config['ssh_host'] = machine_name
-            # "username" => use username@machine_name
-            elif '.' not in arg:
-                config['ssh_host'] = arg + '@' + machine_name
-            # "host-part." => use host-part.machine-domain"
-            elif arg.endswith('.'):
-                config['ssh_host'] = arg + machine_name.split('.', 1)[-1]
-            # "whatever-else" => use "whatever-else"
-            else:
-                config['ssh_host'] = arg
-            del machine_name
-            print('Attempting ssh login with arg: {0}'.format(
-                config['ssh_host']))
-            print('(if you see "Host key verification failed", '
-                  'you need to ssh manually)')
-
         else:
             raise NotImplementedError('Unhandled option', option)
 
@@ -484,14 +440,24 @@ def run_command(command, args, config):
 
     elif command == 'propget':
         # TODO: Add the possibility to replace stdout with a file..
-        fp = pstore.propget(name, property)
+        fp, enctype = pstore.propget(name, property)
+
+        lead = tail = b''
+        if hasattr(os, 'isatty') and os.isatty(sys.stdout.fileno()):
+            # We dropped the last line feed on input from a TTY. So, if
+            # the output is a tty too, we'll add the LF again.
+            if enctype == 'none' or config['verbose']:
+                tail = b'\n'
+            else:
+                # ansi color the password so it's less visible
+                # TODO: invert this if the console is not white-on-black?
+                lead, tail = b'\x1b[7;30m', b'\x1b[0m\n'
+
         # Use a sendfile(2) compatible call to push everything from the fp to
         # stdout.
+        silenced_sigpipe(bytes_stdout.write, lead)
         silenced_sigpipe(sendfile, bytes_stdout, fp)
-        # We dropped the last line feed on input from a TTY. So, if the output
-        # is a tty too, we'll add the LF again.
-        if sys.stdout.isatty():
-            sys.stdout.write('\n')
+        silenced_sigpipe(bytes_stdout.write, tail)
 
     elif command == 'proplist':
         allowed_only = (not config['show_all'])
@@ -517,14 +483,14 @@ def run_command(command, args, config):
 
         if name:
             try:
-                machine_password, machine_info = pstore.get_password(name)
+                properties = pstore.get_object(name)
             except (NotAllowed, NotFound):
                 # A superadmin gets a 404 on no-such-object. A mere mortal gets
                 # a 403 instead. Catch both.
                 pass
             else:
-                run_show_password(machine_info, machine_password,
-                                  ssh_host_hack=config.get('ssh_host', None))
+                silenced_sigpipe(
+                    print_result_properties_and_values, properties)
                 found_machine = True
 
         # If there was no name, or the name wasn't a machine, do a listing.
@@ -595,52 +561,6 @@ def run_command(command, args, config):
         raise NotImplementedError('Unknown command', command)
 
 
-def run_show_password(machine_info, machine_password, ssh_host_hack=None):
-    # FIXME FIXME FIXME
-    # machine_password should be in some kind of mutable XORed form,
-    # so it doesn't easily show up in memory dumps
-    # note that the memory is already exposed earlier on.. go back and fix
-    # it there too.
-    # FIXME FIXME FIXME
-
-    # Er.. why do we assume that the user wants all this on his screen?
-    silenced_sigpipe(print_result_related_properties, machine_info)
-
-    if ssh_host_hack:
-        # Run ssh
-        run_ssh_host_hack(ssh_host_hack)
-    else:
-        # Show password
-        silenced_sigpipe(print_result_password, machine_password)
-
-
-def run_ssh_host_hack(machine_password, ssh_host_hack):
-    if 'DISPLAY' not in os.environ:
-        os.environ['DISPLAY'] = 'whatever:0'
-    os.environ['LD_PRELOAD'] = SSH_ASKPASS_PRELOAD
-    os.environ['SSH_ASKPASS'] = SSH_ASKPASS_APP
-    os.environ['SSH_PASSWORD'] = mktemp()
-    # Not really secure.. using the filesystem.. but it'll have
-    # to do for now..
-    temp = open(os.environ['SSH_PASSWORD'], 'w')
-    try:
-        temp.write(machine_password)
-    finally:
-        temp.close()
-        del temp
-    try:
-        os.execve('/usr/bin/ssh',
-                  ['ssh', '-oPubkeyAuthentication=no', ssh_host_hack],
-                  os.environ)
-    finally:
-        # We don't usually get here, but if we do, make sure we clean up
-        # the password.
-        temp = open(os.environ['SSH_PASSWORD'], 'w')
-        temp.write('X' * 1024)
-        temp.close()
-        os.unlink(os.environ['SSH_PASSWORD'])
-
-
 def silenced_sigpipe(callable, *args, **kwargs):
     try:
         ret = callable(*args, **kwargs)
@@ -687,14 +607,6 @@ def print_results_list(machines):
             ', '.join(sorted(properties['users'].keys()))))
 
 
-def print_result_password(machine_password):
-    if hasattr(os, 'isatty') and os.isatty(sys.stdout.fileno()):
-        # ansi color the password so it's less visible
-        # TODO: invert this if the console is not white-on-black
-        machine_password = b'\x1b[7;30m%s\x1b[0m' % (machine_password,)
-    bytes_stdout.write(b'password = %s\n' % (machine_password,))
-
-
 def print_result_properties(properties, config):
     if config['verbose']:
         for property in sorted(properties.keys()):
@@ -713,10 +625,8 @@ def print_results_property_keys(properties):
         print(objectid)
 
 
-def print_result_related_properties(machine_info):
+def print_result_properties_and_values(machine_info):
     for property in sorted(machine_info['properties'].keys()):
-        if property == PStoreInterface.PASSWORD_PROPERTY:
-            continue  # do that one last..
         info = machine_info['properties'][property]
         if info['enctype'] == 'none':
             if info['data'] is not None:
@@ -764,13 +674,8 @@ def print_result_search(properties, config):
 
 
 def run_usage(verbose=False):
-    # Check if we can use the evil ssh auto-login.
-    evil_ssh_hacks = os.path.exists(SSH_ASKPASS_PRELOAD)
-
     # Select verbosity.
-    if verbose and evil_ssh_hacks:
-        matcher = (lambda i: i in (0, 1, 2))
-    elif verbose:
+    if verbose:
         matcher = (lambda i: i in (0, 1))
     else:
         matcher = (lambda i: i in (0,))
@@ -787,8 +692,6 @@ def run_usage(verbose=False):
          'Look up machine names only (*L)'),
         ('  --propedit=A, -p A    ', 1,
          'Act on properties; list/get/set/encrypt (*P)'),
-        ('  -l [x|SSH_ARG]        ', 2,
-         'Evil hack to ssh(1) to machine directly (*S)'),
         ('  --consistency-check   ', 1,
          'Check database consistency (*C)'),
     )
@@ -820,11 +723,6 @@ def run_usage(verbose=False):
                   '\n(*L) For bash completion. Will never ask for a password.'
                   '\n(*P) Use one of -pl, -pg, -ps, -pe, where -pe is the '
                   'encrypted -ps.')
-    if matcher(2):
-        extra += ('\n(*S) SSH_ARG can be one of "username", '
-                  '"[username@]replace-host.",\n'
-                  '    "[username@]entire-host.name". '
-                  'Requires ssh_askpass.so.')
     if extra:
         extra = '\n' + extra
 
