@@ -69,7 +69,7 @@ class Backend(object):
         path = '/propget/%s/%s.bin' % (urlquote(objectid), urlquote(property))
         out = {u'nonce_b64': self.newnonce(), 'u': self.user}
         reader = self._communicate(path, query=out)
-        return reader.decrypt_with()
+        return reader.decrypt_with(), reader.enctype
 
     def propset(self, objectid, property, files=None, o_excl=False):
         assert files
@@ -211,7 +211,7 @@ class Backend(object):
             first_url = self.urls[0]
             exception = BackendDown('could not connect to %s' % (first_url,))
             exception.__cause__ = first_exception[1]  # PEP 3134 style
-            raise exception  # following form is not python3 compabitle
+            raise exception  # following form is not python3 compatible
             # raise exception, None, first_exception[2]  # pep complaint: W602
 
         return response
@@ -241,6 +241,7 @@ class Backend(object):
             finally:
                 file.close()
             data = None
+            key_expiry = None
 
         # JSON data?
         elif path.endswith('.js'):
@@ -248,6 +249,7 @@ class Backend(object):
                 data = from_jsonfile(utf8_reader(file))
             finally:
                 file.close()
+            key_expiry = None
 
         # Binary data?
         elif path.endswith('.bin'):
@@ -257,9 +259,11 @@ class Backend(object):
             except AttributeError:  # py3
                 content_length = file.headers.get('content-length')
                 enctype = file.headers.get('x-encryption', 'none')
+                key_expiry = file.headers.get('x-key-expiry')
             else:  # py2
                 content_length = file.headers.getheader('content-length')
                 enctype = file.headers.getheader('x-encryption', 'none')
+                key_expiry = file.headers.getheader('x-key-expiry')
             finally:
                 length = int(content_length)
             # No file closing here.. the cryptoreader gets to use it.
@@ -270,6 +274,12 @@ class Backend(object):
             file.close()
             raise NotImplementedError(path)
 
+        # Show warning about keys about to expire.
+        if key_expiry and (int(key_expiry) // 86400) < 90:
+            stderr.write(
+                'WARNING: your (sub)key will expire in %.1f days\n\n' % (
+                    float(key_expiry) / 86400.0))
+
         return data
 
     ###########################################################################
